<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaUser\Controller;

use IMIA\ImiaBaseExt\Utility\BackendTypoScript;
use IMIA\ImiaUser\Utility\Cache;
use PHPExcel_IOFactory;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Messaging\FlashMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * @package     imia_user
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
class BackendController extends AbstractController
{
    /**
     * @var array
     */
    protected $settings;

    /**
     * @var array
     */
    protected $pageData;

    /**
     * @var array
     */
    protected $countries;

    /**
     * @var array
     */
    protected $specialMapping;

    /**
     * @var array
     */
    protected $usernames = array();

    /**
     * @var array
     */
    protected $userFields;

    public function showAction()
    {
        $this->flashMessageContainer->flush();
        $pageData = $this->getPageData();

        if ($pageData['module'] !== 'fe_users') {
            $this->flashMessageContainer->add(
                $this->translate('be_backend_module'),
                null, FlashMessage::INFO
            );
        }

        $userFields = $this->getUserFields();
        $userDisplayFields = array();
        foreach ($userFields as $userField) {
            $userDisplayFields[] = $userField;
        }

        $csvUrl = BackendUtility::getModuleUrl('web_list', array(
            'id' => $pageData['uid'],
            'table' => 'fe_users',
            'imagemode' => 1,
            'csv' => 1,
            'displayFields' => array(
                'fe_users' => $userDisplayFields,
            ),
        ));

        $this->view->assign('userFields', $userFields);
        $this->view->assign('csvUrl', $csvUrl);
        $this->view->assign('pageData', $pageData);
    }

    public function uploadAction()
    {
        $this->flashMessageContainer->flush();
        $pageData = $this->getPageData();

        if ($pageData['module'] !== 'fe_users') {
            $this->redirect('show');
        } else {
            $this->view->assign('pageData', $pageData);
        }
    }

    public function mappingAction()
    {
        $this->flashMessageContainer->flush();
        $pageData = $this->getPageData();

        if ($pageData['module'] !== 'fe_users') {
            $this->redirect('show');
        } else {
            $importData = $this->getImportData($fileName);
            $fields = $this->getFields($importData);

            if (!count($fields)) {
                $this->addError('No data found');
            } else {
                $userFields = $this->getUserFields(true);

                $mapping = $this->getMapping($fields, $userFields);
                $importPreview = array_splice($this->getImportedUsers($fields, $userFields, $importData), 0, 5);

                $this->view->assign('fields', $fields);
                $this->view->assign('fileName', $fileName);
                $this->view->assign('userFields', $userFields);

                $translatedUserFields = array();
                foreach ($userFields as $userField) {
                    $translatedUserFields[$userField] = $this->translate('be_backend_user_' . $userField);
                }
                $this->view->assign('translatedUserFields', $translatedUserFields);

                $this->view->assign('importPreview', $importPreview);
                $this->view->assign('morePreviewCount', count($importData) - 6);
                $this->view->assign('mapping', $mapping);
                $this->view->assign('canImport', count($this->flashMessageContainer->getAllMessages()) > 0 ? false : true);
            }
        }

        $this->view->assign('pageData', $pageData);
    }

    public function importAction()
    {
        $this->flashMessageContainer->flush();
        $pageData = $this->getPageData();

        if ($pageData['module'] !== 'fe_users') {
            $this->redirect('show');
        } else {
            $importData = $this->getImportData($fileName);
            $fields = $this->getFields($importData);

            if (!count($fields)) {
                $this->addError('No data found');
            } else {
                $userFields = $this->getUserFields(true);

                $mapping = $this->getMapping($fields, $userFields);
                $users = $this->getImportedUsers($fields, $userFields, $importData);

                if (count($this->flashMessageContainer->getAllMessages()) == 0) {
                    $imported = array(
                        'new'       => 0,
                        'updated'   => 0,
                    );

                    $persistenceManager = $this->objectManager->get('TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager');
                    foreach ($users as $user) {
                        if ($user->getUid()) {
                            $this->userRepository->update($user);
                            $imported['updated']++;
                        } else {
                            $this->userRepository->add($user);
                            $imported['new']++;
                        }
                        $persistenceManager->persistAll();

                        $recData = array();
                        $recData[$this->getTable()][$user->getUid()]['diable'] = $user->getDisable() ? 1 : 0;

                        $this->initializeTceMain();
                        $this->tce->start($recData, array());
                        $this->tce->process_datamap();
                    }

                    $this->view->assign('imported', $imported);
                } else {
                    $this->redirect('mapping', null, null, array(
                        'fileName'          => $this->request->hasArgument('fileName') ? $this->request->getArgument('fileName') : null,
                        'fields'            => $this->request->hasArgument('field') ? $this->request->getArgument('field') : null,
                        'specialMapping'    => $this->request->hasArgument('specialMapping') ? $this->request->getArgument('specialMapping') : null,
                    ));
                }
            }
        }

        $this->view->assign('pageData', $pageData);
    }

    /**
     * @return array
     */
    public function getSettings()
    {
        if (!$this->settings) {
            $typoScript = BackendTypoScript::get($this->getPageUid());
            $this->settings = $typoScript['plugin.']['tx_imiauser.']['settings.'];
        }

        return $this->settings;
    }

    /**
     * @return array|null
     */
    public function getPageRootUid($pageUid = null)
    {
        if (!$pageUid) {
            $pageUid = $this->getPageUid();
        }

        $page = BackendUtility::getRecordWSOL('pages', $pageUid, 'uid,pid,is_siteroot');
        if (!$page['is_siteroot'] && $page['pid']) {
            $pageUid = $this->getPageRootUid($page['pid']);
        }

        return $pageUid;
    }

    /**
     * @return array|null
     */
    public function getPageData()
    {
        if (!$this->pageData) {
            $this->pageData = BackendUtility::getRecordWSOL('pages', $this->getPageUid(), '*');
            if (!in_array((int)$this->pageData['doktype'], array(199, 254))) {
                $this->pageData['webView'] = true;
            }
        }

        return $this->pageData;
    }

    /**
     * @return integer
     */
    public function getPageUid()
    {
        return (int)GeneralUtility::_GP('id');
    }

    /**
     * @param array $fields
     * @param array $userFields
     * @return array
     */
    protected function getMapping($fields, $userFields)
    {
        $userFieldsStatus = $this->getUserFieldsStatus($fields, $userFields);

        $specialMappingFields = array();
        foreach ($userFieldsStatus as $field => $userFieldStatus) {
            if (!$userFieldStatus['mapped']) {
                if (!$this->isMultipleValuesField($field)) {
                    $specialMapping = $this->getSpecialMapping($field);
                    $specialMappingFields[$field] = array(
                        'field'     => $field,
                        'label'     => $this->translate('be_backend_user_' . strtolower($field)),
                        'value'     => $specialMapping ?: null,
                    );

                    if ($specialMapping) {
                        $userFieldsStatus[$field]['mapped'] = $userFieldStatus['mapped'] = true;
                    }
                }

                if ($userFieldStatus['required'] && !$userFieldStatus['mapped']) {
                    $this->addError('required field ' . $field . ' not mapped');
                }
            }
        }

        $mappingUserFields = array('');
        $specialMappingOptions = array(
            'default' => array()
        );

        foreach ($userFields as $userField) {
            $mappingUserFields[$userField] = $this->translate('be_backend_user_' . strtolower($userField)) ?: $userField;

            if (!array_key_exists($userField, $specialMappingFields) && !$this->isMultipleValuesField($userField)) {
                $specialMappingOptions['default'][$userField] = $this->translate('be_backend_specialmapping_infield') . $mappingUserFields[$userField];
            }
        }
        asort($mappingUserFields);

        $specialMappingOptions['password'] = array('',
            'password_generate' => $this->translate('be_backend_specialmapping_passwordgenerate')) +
            $specialMappingOptions['default'];

        $specialMappingOptions['default'] = array('') +
            $specialMappingOptions['default'];

        // Hook: getMappingFields
        foreach ($this->getHookObjects() as $hookObject) {
            if (method_exists($hookObject, 'getMappingFields')) {
                $hookObject->getMappingFields($userFields, $mappingUserFields, $specialMappingFields,
                    $specialMappingOptions, $this);
            }
        }

        foreach ($specialMappingFields as $key => $specialMappingField) {
            $specialMappingFields[$key]['options'] = array_key_exists($key, $specialMappingOptions)
                ? $specialMappingOptions[$key]
                : $specialMappingOptions['default'];

            asort($specialMappingFields[$key]['options']);
        }

        return array(
            'userfields'        => $mappingUserFields,
            'userfieldsStatus'  => $userFieldsStatus,
            'specialfields'     => $specialMappingFields,
        );
    }

    /**
     * @param array $fields
     * @param array $importData
     * @return array
     */
    protected function getUserFieldsStatus($fields, $userFields)
    {
        $userFieldsStatus = array();
        foreach ($userFields as $userField) {
            $userFieldsStatus[$userField] = array(
                'mapped'    => false,
                'required'  => in_array($userField, array('username', 'password', 'email')) ? true : false,
            );
        }

        foreach ($fields as $field) {
            if ($field['userfield'] && array_key_exists($field['userfield'], $userFieldsStatus)) {
                $userFieldsStatus[$field['userfield']]['mapped'] = true;
            }
        }

        return $userFieldsStatus;
    }

    /**
     *
     * @param string $field
     * @return string
     */
    protected function getSpecialMapping($field = null)
    {
        if (!$this->specialMapping) {
            if ($this->request->hasArgument('specialMapping') && is_array($this->request->getArgument('specialMapping'))) {
                $this->specialMapping = $this->request->getArgument('specialMapping');
            }
        }

        if ($field) {
            return $this->specialMapping && is_array($this->specialMapping) && array_key_exists($field, $this->specialMapping) && $this->specialMapping[$field]
                ? $this->specialMapping[$field]
                : null;
        } else {
            return $this->specialMapping;
        }
    }

    /**
     * @param array $fields
     * @param array $importData
     * @param array $userFields
     * @param integer $limit
     * @return array
     */
    protected function getImportedUsers($fields, $userFields, $importData)
    {
        $users = array();
        array_shift($importData);

        foreach ($importData as $importRow) {
            $row = array();
            foreach ($userFields as $userField) {
                $row[$userField] = null;
            }

            foreach ($importRow as $key => $importValue) {
                if (array_key_exists($key, $fields)) {
                    $userField = $fields[$key]['userfield'];
                    if ($userField) {
                        $value = $row[$userField];

                        if ($value) {
                            if ($this->isMultipleValuesField($userField)) {
                                if ($fields[$key]['valueMap']) {
                                    foreach ($fields[$key]['valueMap']['values'] as $fieldValue) {
                                        if ($fieldValue['title'] == trim($importValue) && $fieldValue['option']) {
                                            $value .= ',' . $fieldValue['option'];
                                            break;
                                        }
                                    }
                                } else {
                                    $value .= ',' . $importValue;
                                }
                            } else {
                                $this->addError('error: field ' . $userField . ' with duplicate value');
                            }
                        } else {
                            if ($fields[$key]['valueMap']) {
                                foreach ($fields[$key]['valueMap']['values'] as $fieldValue) {
                                    if ($fieldValue['title'] == trim($importValue)) {
                                        $value = $fieldValue['option'];
                                        break;
                                    }
                                }
                            } else {
                                $value = $importValue;
                            }
                        }

                        $row[$userField] = $value;
                    }
                }
            }

            $specialMapping = $this->getSpecialMapping();
            if ($specialMapping && is_array($specialMapping)) {
                foreach ($specialMapping as $field => $option) {
                    if ($option == 'password_generate') {
                        $row[$field] = substr(sha1(uniqid(md5(rand()), true)), 0, 12);
                    } elseif ($row[$option]) {
                        $row[$field] = $row[$option];
                    }
                }
            }

            $users[] = $this->createUser($row);
        }

        return $users;
    }

    /**
     * @param array $row
     * @return \IMIA\ImiaUser\Domain\Model\User
     */
    protected function createUser($row)
    {
        if ($this->settings['emailAsUsername']) {
            $row['username'] = $row['email'];
        }

        $user = $this->userRepository->findOneByUsername($row['username']);
        if (!$user) {
            $user = $this->userRepository->findOneByEmail($row['email']);
            if (!$user) {
                $user = $this->objectManager->create('IMIA\ImiaUser\Domain\Model\User');
            }
        }

        // Hook: createUser
        foreach ($this->getHookObjects() as $hookObject) {
            if (method_exists($hookObject, 'createUser')) {
                $hookObject->createUser($user, $row, $this->userRepository, $this);
            }
        }

        foreach ($row as $field => $value) {
            if ($value !== null) {
                $methodName = 'set' . GeneralUtility::underscoredToUpperCamelCase($field);

                if (method_exists($user, $methodName)) {
                    switch ($field) {
                        case 'country':
                            $country = $this->countryRepository->findOneByUid($value);
                            if ($country) {
                                $user->$methodName($country);
                            }
                            break;
                        case 'password':
                            if (!$user->getUid() || $this->getSpecialMapping($field) != 'password_generate') {
                                $user->$methodName($value);
                            }
                            break;
                        case 'username':
                            if (in_array($value, $this->usernames)) {
                                $this->addError('Duplicate username: ' . $value);
                            } else {
                                $this->usernames[] = $value;
                            }

                            $user->$methodName($value);
                            break;
                        default:
                            $user->$methodName($value);
                        break;
                    }
                }
            }
        }

        // Hook: updateUser
        foreach ($this->getHookObjects() as $hookObject) {
            if (method_exists($hookObject, 'updateUser')) {
                $hookObject->updateUser($row, $user, $this);
            }
        }

        return $user;
    }

    /**
     * @param string $userField
     * @return boolean
     */
    protected function isMultipleValuesField($userField)
    {
        switch ($userField) {
            default:
                $multiple = false;
                break;
        }

        // Hook: isMultipleValuesField
        foreach ($this->getHookObjects() as $hookObject) {
            if (method_exists($hookObject, 'isMultipleValuesField')) {
                $hookObject->isMultipleValuesField($userField, $multiple, $this);
            }
        }

        return $multiple;
    }

    /**
     * @param array $importData
     * @return array
     */
    protected function getFields($importData)
    {
        $fields = array();
        $fieldRow = array_shift($importData);
        $fieldData = array_values($importData);

        $mappingKey = sha1(implode(',', $fieldRow) . '_' . implode(',', array_keys($fieldRow)));
        $cache = new Cache('imia_user-import-' . $mappingKey, true);

        $mapping = null;
        if ($this->request->hasArgument('field') && is_array($this->request->getArgument('field'))){
            $mapping = $this->request->getArgument('field');

            $cache->write(array(
                $mapping,
                $this->getSpecialMapping())
            );
        } else {
            if ($cache->exists()) {
                list($mapping, $this->specialMapping) = $cache->load();
            }
        }

        $tcaColumns = $GLOBALS['TCA'][$this->getTable()]['columns'];
        foreach ($fieldRow as $key => $fieldName) {
            if (trim($fieldName)) {
                $fields[$key] = array(
                    'title'     => $fieldName,
                    'example'   => $importData[0][$key],
                    'userfield' => is_array($mapping) ? $mapping[$key]['userfield'] : $this->guessMapping($fieldName),
                    'valueMap'  => false,
                );

                $field = &$fields[$key];
                if (!in_array($field['userfield'], $this->getUserFields(true))) {
                    $field['userfield'] = '';
                }

                switch ($field['userfield']) {
                    case 'country':
                        $countryOptions = array('');
                        foreach ($this->getCountries() as $country) {
                            $countryOptions[$country->getUid()] = ($GLOBALS['LANG']->lang == 'de' && $country->getShortDe())
                                ? $country->getShortDe()
                                : $country->getShortEn();
                        }
                        asort($countryOptions);

                        $field['valueMap'] = array(
                            'options'   => $countryOptions,
                        );
                        break;
                    default:
                        $tcaColumnConfig = $tcaColumns[$field['userfield']]['config'];
                        if (in_array($tcaColumnConfig['type'], array('select', 'radio'))) {
                            $field['valueMap'] = array('options' => array());
                            foreach ($tcaColumnConfig['items'] as $item) {
                                $field['valueMap']['options'][$item[1]] = $this->translate($item[0]);
                            }
                        }
                        break;
                }

                // Hook: getFieldSettings
                foreach ($this->getHookObjects() as $hookObject) {
                    if (method_exists($hookObject, 'getFieldSettings')) {
                        $hookObject->getFieldSettings($field, $this);
                    }
                }
            }
        }

        foreach ($fieldData as $data) {
            foreach ($data as $key => $value) {
                if ($fields[$key]['valueMap']) {
                    if (!$fields[$key]['valueMap']['values']) {
                        $fields[$key]['valueMap']['values'] = array();
                    }
                    $fields[$key]['valueMap']['values'][] = trim($value);
                }
            }
        }
        foreach ($fields as $key => $val) {
            if ($fields[$key]['valueMap']) {
                $values = array_unique($fields[$key]['valueMap']['values']);
                sort($values);

                $fields[$key]['valueMap']['values'] = array();
                foreach ($values as $value) {
                    $fields[$key]['valueMap']['values'][] = array(
                        'title'     => $value,
                        'option'    => is_array($mapping) && $mapping[$key]['option'] ? $mapping[$key]['option'][$value] :
                            $this->guessMapping($fields[$key]['title'], $value),
                    );
                }
            }
        }

        return $fields;
    }

    /**
     * @param string $fileName
     * @return array
     */
    protected function getImportData(&$fileName)
    {
        $importData = null;
        $tempPath = PATH_site . 'typo3temp/imia_user';
        if (!is_dir($tempPath)) {
            @mkdir($tempPath);
        }

        $fileData = $this->getFileArgument('file');
        if ($fileData) {
            $fileName = $fileData['name'];
            $filePath =  $tempPath . '/' . $fileName;
            if (!@copy($fileData['tmp_name'], $filePath)) {
                unset($filePath);
                $this->addError('Writing uploaded file failed');
            }
        } elseif ($this->request->hasArgument('fileName')) {
            $fileName = $this->request->getArgument('fileName');
            $filePath = $tempPath . '/' . $fileName;

            if (!file_exists($filePath)) {
                unset($filePath);
                $this->addError('File not found');
            }
        }

        if ($filePath) {
            try {
                $importData = PHPExcel_IOFactory::load($filePath)
                    ->getActiveSheet()
                    ->toArray(null, true, true, true);

            } catch (\Exception $e) {
                $this->addError($e->getMessage());
            }
        }

        return $importData;
    }

    /**
     * @param string $field
     * @param string $value
     * @return string
     */
    protected function guessMapping($field, $value = null)
    {
        switch (strtolower(trim($field))) {
            case 'country':
            case 'land':
                $field = 'country';
                break;
            case 'city':
            case 'town':
            case 'stadt':
            case 'ort':
            case 'wohnort':
                $field = 'city';
                break;
            case 'salutation':
            case 'gender':
            case 'anrede':
            case 'geschlecht':
                $field = 'gender';
                break;
            case 'first name':
            case 'firstname':
            case 'vorname':
                $field = 'first_name';
                break;
            case 'last name':
            case 'lastname':
            case 'name':
            case 'family name':
            case 'nachname':
            case 'name':
            case 'familienname':
                $field = 'last_name';
                break;
            case 'email':
            case 'e-mail':
            case 'mail':
                $field = 'email';
                break;
            case 'telephone':
            case 'phone':
            case 'telefon':
                $field = 'telephone';
                break;
            case 'username':
            case 'login':
            case 'benutzername':
                $field = 'username';
                break;
            case 'address':
            case 'adresse':
            case 'anschrift':
            case 'straße':
            case 'strasse':
                $field = 'address';
                break;
            case 'plz':
            case 'postcode':
            case 'zip':
                $field = 'zip';
                break;
            case 'fax':
            case 'telefax':
                $field = 'fax';
                break;
            case 'mobile':
            case 'mobil':
            case 'mobile phone':
            case 'handy':
                $field = 'mobile';
                break;
            case 'company':
            case 'firma':
            case 'unternehmen':
            case 'unternehmung':
            case 'partner':
                $field = 'company';
                break;
            case 'position':
                $field = 'position';
                break;
            case 'password':
            case 'passwort':
            case 'kennwort':
                $field = 'password';
                break;
            default:
                $field = '';
                break;
        }

        if ($field && $value) {
            $option = '';
            switch ($field) {
                default:
                    case 'gender':
                        switch (strtolower(trim($value))) {
                            case 'male':
                            case 'männlich':
                            case 'mann':
                            case 'man':
                            case 'mr.':
                            case 'mr':
                            case 'herr':
                                $option = 'm';
                                break;
                            case 'female':
                            case 'woman':
                            case 'mrs.':
                            case 'mrs':
                            case 'frau':
                                $option = 'f';
                                break;
                        }
                        break;
                    case 'country':
                        foreach ($this->getCountries() as $country) {
                            if (in_array(strtolower(trim($value)), array(
                                strtolower($country->getShortEn()),
                                strtolower($country->getShortDe()),
                                strtolower($country->getShortLocal()),
                                strtolower($country->getOfficialNameLocal()),
                                strtolower($country->getOfficialNameEn()),
                              ))) {
                                $option = $country->getUid();
                                break;
                            }
                        }
                        break;
            }

            return $option;
        } else {
            return $field;
        }
    }

    /**
     * @return array
     */
    protected function getCountries()
    {
        if (!$this->countries) {
            $this->countries = $this->countryRepository->findAll();
        }

        return $this->countries;
    }

    /**
     * @return array
     */
    protected function getUserFields($password = false)
    {
        if (!$this->userFields) {
            $this->userFields = array('username', 'password');

            $settings = $this->getSettings();
            foreach (explode(',', $settings['fields.']['enabled']) as $field) {
                $this->userFields[] = GeneralUtility::camelCaseToLowerCaseUnderscored(trim($field));
            }

            // Hook: getUserFields
            foreach ($this->getHookObjects() as $hookObject) {
                if (method_exists($hookObject, 'getUserFields')) {
                    $hookObject->getUserFields($this->userFields, $password, $this);
                }
            }
        }

        if (!$password && in_array('password', $this->userFields)) {
            $this->userFields = array_diff(array('password'), $this->userFields);
        }

        if ($this->settings['emailAsUsername']) {
            $ufc = count($this->userFields);
            $this->userFields = array_diff($this->userFields, array('username'));
            if ($ufc < count($this->userFields)) {
                $this->userFields[] = 'email';
            }
        }

        return array_unique($this->userFields);
    }
}