<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaUser\Controller;

use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use IMIA\ImiaUser\Domain\Model\User;

/**
 * @package     imia_user
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
class UserController extends AbstractController
{
    /**
     * @var User
     */
    protected $user;

    /**
     * @var array
     */
    protected $fields;

    public function accessAction()
    {
        $user = $this->getUser(true);
        $this->flashMessageContainer->flush();

        if ($this->request->getMethod() == 'POST') {
            if ($this->request->hasArgument('password') && $this->request->getArgument('password')) {
                if (strlen($this->request->getArgument('password')) < 6) {
                    $this->addError('user_form_error_password_length', 'password');
                } elseif ($this->getSetting('passwordRepeat') && (!$this->request->hasArgument('passwordRepeat') || $this->request->getArgument('password') != $this->request->getArgument('passwordRepeat'))) {
                    $this->addError('user_form_error_password_repeat', 'password_repeat');
                } else {
                    $user->setPassword($this->request->getArgument('password'));
                    $this->userRepository->update($user);

                    $this->addSuccess('user_access_form_success_password');
                }
            } elseif ($this->request->hasArgument('newPassword') && $this->request->getArgument('newPassword')) {
                if (strlen($this->request->getArgument('newPassword')) < 6) {
                    $this->addError('user_form_error_password_length', 'new_password');
                } elseif ($this->getSetting('passwordRepeat') && (!$this->request->hasArgument('newPasswordRepeat') || $this->request->getArgument('newPassword') != $this->request->getArgument('newPasswordRepeat'))) {
                    $this->addError('user_form_error_password_repeat', 'new_password_repeat');
                } else {
                    $user->setPassword($this->request->getArgument('newPassword'));
                    $this->userRepository->update($user);

                    $this->addSuccess('user_access_form_success_password');
                }
            }

            if ($this->settings['emailAsUsername']) {
                if ($this->request->hasArgument('newEmail') && $this->request->getArgument('newEmail')) {
                    $newEmail = $this->request->getArgument('newEmail');

                    if (!GeneralUtility::validEmail($newEmail)) {
                        $this->addError('user_form_error_email_valid', 'new_email');
                    } else {
                        $otherUser = $this->userRepository->findOneByEmail($newEmail);
                        if ($otherUser && (!$user || $otherUser->getUid() != $user->getUid())) {
                            $this->addError('user_form_error_email_duplicate', 'new_email');
                        }

                        if (!$otherUser) {
                            $otherUser = $this->userRepository->findOneByUsername($newEmail);
                            if ($otherUser && (!$user || $otherUser->getUid() != $user->getUid())) {
                                $this->addError('user_form_error_email_duplicate', 'new_email');
                            }
                        }

                        if (!$otherUser) {
                            $user->setEmail($newEmail);
                            $user->setUsername($newEmail);
                            $this->userRepository->update($user);

                            $this->addSuccess('user_access_form_success_email');
                        }
                    }
                }
            } else {
                if ($this->request->hasArgument('newUsername') && $this->request->getArgument('newUsername')) {
                    $newUsername = $this->request->getArgument('newUsername');

                    if ($this->userRepository->findOneByUsername($newUsername)) {
                        $this->addError('user_form_error_username_duplicate', 'new_username');
                    } elseif (strlen($user->getUsername()) < 4) {
                        $this->addError('user_form_error_username_length', 'new_username');
                    } else {
                        $user->setUsername($newUsername);
                        $this->userRepository->update($user);

                        $this->addSuccess('user_access_form_success_username');
                    }
                }
            }
        }

        $this->view->assign('user', $user);
        $this->view->assign('cobj', $this->configurationManager->getContentObject()->data);
    }

    public function profileAction()
    {
        $user = $this->getUser(true);
        $this->flashMessageContainer->flush();

        $error = false;
        if ($this->request->getMethod() == 'POST') {
            if ($this->validate($user)) {
                if ($this->settings['gtc'] && $this->settings['gtcEditable']) {
                    if (!$this->settings['gtcOptional'] && (!$this->request->hasArgument('gtc') || !$this->request->getArgument('gtc'))) {
                        if (!$user->getGtc()) {
                            $this->addError('user_form_error_gtc', 'gtc');
                            $error = true;
                        }
                    } elseif ($this->request->hasArgument('gtc')) {
                        $user->setGtc((bool)$this->request->getArgument('gtc'));
                    }
                }

                if (!$error) {
                    $this->uploadFiles($user);
                    $this->userRepository->update($user);
                    $this->clearCache();

                    $this->addSuccess('user_profile_form_success');
                }
            }
        }

        if (!$this->settings['gtcOptional'] && $user->getGtc()) {
            $this->settings['gtcEditable'] = false;
            $this->view->assign('settings', $this->settings);
        }

        $this->view->assign('user', $user);
        $this->view->assign('fields', $this->getFields());
        $this->view->assign('info', $this->getInfo());
        $this->view->assign('cobj', $this->configurationManager->getContentObject()->data);
    }

    public function registerAction()
    {
        $this->flashMessageContainer->flush();

        $user = $this->objectManager->create('IMIA\ImiaUser\Domain\Model\User');
        $user->generateConfirmToken();
        $user->setLanguage($this->getCurrentLanguage());
        $user->setDisable(true);

        if ($this->settings['captcha']['_typoScriptNodeValue']) {
            @require_once(ExtensionManagementUtility::extPath('imia_user') . 'Classes/Lib/recaptchalib.php');
        }

        $registerChecks = $this->getSetting('addRegisterCheck');
        if (!$registerChecks || !is_array($registerChecks)) {
            $registerChecks = array();
        }

        $checkCount = 0;
        foreach ($registerChecks as $key => $registerCheck) {
            $registerChecks[$key]['id'] = 'addcheck-' . ++$checkCount;
        }
        $this->view->assign('registerChecks', $registerChecks);

        $captchaError = null;
        $formError = false;
        if ($this->request->getMethod() == 'POST') {
            $validateSuccess = $this->validate($user, true);

            if ($this->settings['captcha']['_typoScriptNodeValue']) {
                $captchaResponse = recaptcha_check_answer($this->settings['captcha']['privateKey'],
                        $_SERVER['REMOTE_ADDR'], $_POST['recaptcha_challenge_field'], $_POST['recaptcha_response_field']);

                if (!$captchaResponse->is_valid) {
                    $captchaError = $captchaResponse->error;
                    $this->addError('user_form_error_captcha', 'captcha');
                }
            }

            if (count($registerChecks) > 0) {
                foreach ($registerChecks as $registerCheck) {
                    if ($registerCheck['mandatory'] && $registerCheck['error']) {
                        if (!$this->request->hasArgument($registerCheck['id']) || !$this->request->getArgument($registerCheck['id'])) {
                            $formError = true;
                            $this->addError($registerCheck['error'], $registerCheck['id']);
                        }
                    }
                }
            }

            if ($this->settings['gtc'] && !$this->settings['gtcOptional']) {
                if (!$this->request->hasArgument('gtc') || !$this->request->getArgument('gtc')) {
                    $formError = true;
                    $this->addError('user_form_error_gtc', 'gtc');
                } else {
                    $user->setGtc(true);
                }
            }

            if ($validateSuccess && !$captchaError && !$formError) {
                if ($this->settings['usergroup']) {
                    $user->setUsergroup((int)$this->settings['usergroup']);
                }

                $this->uploadFiles($user);
                $this->userRepository->add($user);
                $this->objectManager->get('TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager')->persistAll();

                $recData = array();
                $recData[$this->getTable()][$user->getUid()]['diable'] = $user->getDisable() ? 1 : 0;

                $this->initializeTceMain();
                $this->tce->start($recData, array());
                $this->tce->process_datamap();

                $this->clearCache();

                $this->sendRegistrationMail($user, 'preConfirm');

                return $this->forward('registerSuccess', null, null, array('user' => $user));
            }
        }

        if ($this->settings['captcha']['_typoScriptNodeValue']) {
            $this->view->assign('captcha', recaptcha_get_html($this->settings['captcha']['publicKey'], $captchaError));
        }

        $this->view->assign('user', $user);
        $this->view->assign('fields', $this->getFields());
        $this->view->assign('info', $this->getInfo());
        $this->view->assign('cobj', $this->configurationManager->getContentObject()->data);
    }

    /**
     * @param string $token
     */
    public function confirmAction($token)
    {
        $user = $this->userRepository->findOneByConfirmToken($token);

        if ($user) {
            $user->clearConfirmToken();

            if (!$this->settings['activation']) {
                $user->setDisable(false);
                $user->clearActivateToken();

                $this->userRepository->update($user);
                $this->objectManager->get('TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager')->persistAll();

                $this->loginUser($user->getUsername(), $user->getPlainPassword(), true);
                $this->sendRegistrationMail($user, 'confirmed');
            } else {
                $user->setDisable(true);
                $user->generateActivateToken();

                $this->sendRegistrationMail($user, 'confirmPreActivation');
                $this->sendRegistrationMail($user, 'new', 'admin');
            }
            $this->userRepository->update($user);

            return $this->forward('confirmSuccess', null, null, array('user' => $user));
        }
    }

    /**
     * @param string $token
     */
    public function activateAction($token)
    {
        if ($this->settings['activation'] && $GLOBALS['BE_USER'] && ($GLOBALS['BE_USER']->user['admin']
              || $GLOBALS['BE_USER']->isInWebMount($GLOBALS['TSFE']->id))) {
            $user = $this->userRepository->findOneByActivateToken($token);

            if ($user) {
                $user->setDisable(false);
                $user->clearActivateToken();
                $this->userRepository->update($user);

                $this->sendRegistrationMail($user, 'confirmActivation');

                return $this->forward('activateSuccess', null, null, array('user' => $user));
            }
        } else {
            throw(new \Exception('Bitte melden Sie sich in TYPO3 an um einen Benutzer freizuschalten', 403));
        }
    }

    /**
     * @param User $user
     */
    public function registerSuccessAction(User $user)
    {
        $this->view->assign('user', $user);
        $this->view->assign('cobj', $this->configurationManager->getContentObject()->data);
    }

    /**
     * @param User $user
     */
    public function confirmSuccessAction(User $user)
    {
        $this->view->assign('user', $user);
        $this->view->assign('cobj', $this->configurationManager->getContentObject()->data);
    }

    /**
     * @param User $user
     */
    public function activateSuccessAction(User $user)
    {
        $this->view->assign('user', $user);
        $this->view->assign('cobj', $this->configurationManager->getContentObject()->data);
    }

    public function deleteAction()
    {
        $user = $this->getUser(true);
        $this->flashMessageContainer->flush();

        if ($this->request->getMethod() == 'POST') {
            $this->userRepository->remove($user);
            return $this->forward('deleteSuccess', null, null, array('user' => $user));
        }

        $this->view->assign('user', $user);
        $this->view->assign('cobj', $this->configurationManager->getContentObject()->data);
    }

    /**
     * @param User $user
     */
    public function deleteSuccessAction(User $user)
    {
        $this->view->assign('user', $user);
        $this->view->assign('cobj', $this->configurationManager->getContentObject()->data);
    }

    /**
     * @param User $user
     * @param boolean $registration
     * @return boolean
     */
    protected function validate(User &$user, $registration = false)
    {
        $fields = $this->getFields();
        if ($registration) {
            array_unshift($fields['enabled'], 'password');

            if ($this->settings['emailAsUsername']) {
                array_unshift($fields['enabled'], 'email');
            } else {
                array_unshift($fields['enabled'], 'username');
            }
        }

        foreach ($fields['enabled'] as $field) {
            $setter = 'set' . ucfirst($field);
            $getter = 'get' . ucfirst($field);

            switch ($field) {
                case 'username':
                    if ($this->request->hasArgument($field)) {
                        $user->$setter($this->request->getArgument($field));
                    }

                    if (!$user->$getter()) {
                        $this->addError('user_form_error_' . strtolower($field), strtolower($field));
                    } else {
                        if ($this->userRepository->findOneByUsername($this->request->getArgument($field))) {
                            $this->addError('user_form_error_' . strtolower($field) .'_duplicate', strtolower($field));
                        } elseif (strlen($user->$getter()) < 4) {
                            $this->addError('user_form_error_username_length', 'username');
                        }
                    }
                    break;
                case 'password':
                    if ($this->request->hasArgument($field)) {
                        $user->$setter($this->request->getArgument($field));
                    }

                    if ($this->request->hasArgument('password')) {
                        if (!$user->getPlainPassword()) {
                            $this->addError('user_form_error_password', 'password');
                        } elseif (strlen($user->getPlainPassword()) < 6) {
                            $this->addError('user_form_error_password_length', 'password');
                        } elseif ($this->getSetting('passwordRepeat') && (!$this->request->hasArgument('passwordRepeat') || $user->getPlainPassword() != $this->request->getArgument('passwordRepeat'))) {
                            $this->addError('user_form_error_password_repeat', 'password_repeat');
                        }
                    }
                    break;
                case 'email':
                    if ($this->request->hasArgument($field)) {
                        $user->$setter($this->request->getArgument($field));
                    }

                    $otherUser = null;
                    if ((in_array($field, $fields['mandatory']) || $this->settings['emailAsUsername']) && !$user->$getter()) {
                        $this->addError('user_form_error_' . strtolower($field), strtolower($field));
                    } elseif (!GeneralUtility::validEmail($user->$getter())) {
                        $this->addError('user_form_error_' . strtolower($field) . '_valid', strtolower($field));
                    } else {
                        $otherUser = $this->userRepository->findOneByEmail($user->$getter());

                        if ($otherUser && (!$user || $otherUser->getUid() != $user->getUid())) {
                            $this->addError('user_form_error_' . strtolower($field) .'_duplicate', strtolower($field));
                        }
                    }

                    if ($this->settings['emailAsUsername']) {
                        $usernameSetter = 'setUsername';
                        $user->$usernameSetter($user->$getter());

                        if (!$otherUser && $this->request->hasArgument($field)) {
                            $otherUser = $this->userRepository->findOneByUsername($this->request->getArgument($field));
                            if ($otherUser && (!$user || $otherUser->getUid() != $user->getUid())) {
                                $this->addError('user_form_error_' . strtolower($field) .'_duplicate', strtolower($field));
                            }
                        }
                    }

                    break;
                case 'country':
                    $country = null;
                    if ($this->request->hasArgument($field)) {
                        $country = $this->countryRepository->findOneByUid((int)$this->request->getArgument($field));
                    }
                    $user->$setter($country);

                    if (in_array($field, $fields['mandatory']) && !$user->$getter()) {
                        $this->addError('user_form_error_' . strtolower($field), strtolower($field));
                    }
                    break;
                case 'language':
                    $language = null;
                    if ($this->request->hasArgument($field)) {
                        $language = $this->languageRepository->findOneByUid((int)$this->request->getArgument($field));
                    }
                    $user->$setter($language);

                    if (in_array($field, $fields['mandatory']) && !$user->$getter()) {
                        $this->addError('user_form_error_' . strtolower($field), strtolower($field));
                    }
                    break;
                case 'image':
                    $fileData = $this->getFileArgument($field);
                    if ($fileData) {
                        if (strpos($fileData['type'], 'image') === false) {
                            $this->addError('user_form_error_' . strtolower($field) . '_valid', strtolower($field));
                        } elseif ($fileData['size'] / 1024 > 2 * 1024) {
                            $this->addError('user_form_error_' . strtolower($field) . '_size', strtolower($field));
                        }
                    } elseif (in_array($field, $fields['mandatory'])) {
                        $this->addError('user_form_error_' . strtolower($field), strtolower($field));
                    }
                    break;
                case 'pdf':
                    $fileData = $this->getFileArgument($field);
                    if ($fileData) {
                        if (strpos($fileData['type'], 'pdf') === false) {
                            $this->addError('user_form_error_' . strtolower($field) . '_valid', strtolower($field));
                        } elseif ($fileData['size'] / 1024 > 5 * 1024) {
                            $this->addError('user_form_error_' . strtolower($field) . '_size', strtolower($field));
                        }
                    } elseif (in_array($field, $fields['mandatory'])) {
                        $this->addError('user_form_error_' . strtolower($field), strtolower($field));
                    }
                    break;
                default:
                    if ($this->request->hasArgument($field)) {
                        $user->$setter($this->request->getArgument($field));
                    }

                    if (in_array($field, $fields['mandatory']) && !$user->$getter()) {
                        $this->addError('user_form_error_' . strtolower($field), strtolower($field));
                    }
                    break;
            }
        }

        return count($this->flashMessageContainer->getAllMessages()) == 0 ? true : false;
    }

    /**
     * @param User $user
     */
    protected function uploadFiles(User &$user)
    {
        $fields = $this->getFields();
        $uploadFields = array('image', 'pdf');
        foreach ($uploadFields as $field) {
            if (in_array($field, $fields['enabled'])) {
                if ($this->request->hasArgument($field . '_delete') && $this->request->getArgument($field . '_delete')) {
                    $deleteMethod = 'remove' . ucfirst($field);
                    $user->$deleteMethod();
                } else {
                    $fileData = $this->getFileArgument($field);
                    if ($fileData) {
                        $setMethod = 'set' . ucfirst($field);
                        $folder = 'community/user/' . $user->getUid() . '_' . $user->getLastName() ?: $user->getCompany() . '/';
                        $file = $this->uploadFile($fileData, (int)$this->settings['file']['storageUid'], $folder);
                        $user->$setMethod($file);
                    }
                }
            }
        }
    }

    /**
     * @param User $user
     * @param string $mail
     * @param string $mailType
     */
    protected function sendRegistrationMail($user, $mail, $mailType = null)
    {
        if ($mailType) {
            $mailSettings = $this->settings['email'][$mailType];
        } else {
            $mailSettings = $this->settings['email'];
        }

        if ($mailSettings['recipientMail']) {
            $recipient = array($mailSettings['recipientMail']);
        } else {
            $recipient = array($user->getEmail());
        }

        $emailFormat = $mailSettings['type'];
        if (!in_array($emailFormat, array('plain', 'html'))) {
            $emailFormat = 'plain';
        }

        if ($mailSettings['senderName']) {
            $sender = array($mailSettings['senderMail'] => $mailSettings['senderName']);
        } else {
            $sender = array($mailSettings['senderMail']);
        }

        $templatePath = PATH_site . 'typo3temp/Cache/Code/cache_core/imia_user/' .
            $this->configurationManager->getContentObject()->data['uid'] . '_' . $mail . '_' . ($mailType ? $mailType . '_' : '') .
            $emailFormat . '.txt';

        if (!is_dir(dirname($templatePath))) {
            @mkdir(dirname($templatePath), 0777, true);
            @exec('chmod 777 -R ' . PATH_site . 'typo3temp');
        }

        $templateContent = $mailSettings[$mail . 'Text'][$emailFormat];
        @file_put_contents($templatePath, $templateContent);

        if (file_exists($templatePath)) {
            $this->sendMail(
                $recipient,
                $sender,
                $mailSettings[$mail . 'Subject'],
                $this->renderTemplate($templatePath, array(
                    'user'          => $user,
                    'loginUrl'      => !$user->getDisable() && $mailType != 'admin' && $this->settings['pages']['loginUid']
                        ? $this->uriBuilder->setCreateAbsoluteUri(true)->setTargetPageUid($this->settings['pages']['loginUid'])->build()
                        : '',
                    'confirmUrl'    => !$user->getConfirmed()
                        ? $this->uriBuilder->setCreateAbsoluteUri(true)->setTargetPageUid($GLOBALS['TSFE']->id)->uriFor('confirm', array('token' => $user->getConfirmToken()))
                        : '',
                    'activationUrl' => $user->getDisable() && $mailType == 'admin'
                        ? $this->uriBuilder->setCreateAbsoluteUri(true)->setTargetPageUid($GLOBALS['TSFE']->id)->uriFor('activate', array('token' => $user->getActivateToken()))
                        : '',
                )),
                $emailFormat
            );
        }
    }

    protected function clearCache()
    {
        if ($this->settings['cache']['pids']) {
            if (is_array($this->settings['cache']['pids'])) {
                $pIds = $this->configurationManager->getContentObject()->cObjGetSingle(
                    $this->settings['cache']['pids']['_typoScriptNodeValue'],
                    $this->settings['cache']['pids']);
            } else {
                $pIds = $this->settings['cache']['pids'];
            }
            $pageUids = array_map('intval', explode(',', trim($pIds)));

            $this->cacheService->clearPageCache($pageUids);
        }
    }

    /**
     * @return array
     */
    protected function getInfo()
    {
        $countries = array('');
        foreach ($this->countryRepository->findAll() as $country) {
            $countries[$country->getUid()] = (string)$country;
        }
        asort($countries);

        $languages = array('');
        foreach ($this->languageRepository->findAll() as $language) {
            $languages[$language->getUid()] = (string)$language;
        }
        asort($languages);

        return array(
            'countries' => $countries,
            'languages' => $languages,
        );
    }

    /**
     * @return \IMIA\ImiaUser\Domain\Model\Language
     */
    protected function getCurrentLanguage()
    {
        if ((int)$this->settings['defaultLanguage'] > 0) {
            $languageUid = (int)$this->settings['defaultLanguage'];
        } else {
            $languageUid = $GLOBALS['TSFE']->sys_language_uid;
        }

        return $this->languageRepository->findOneByUid($languageUid);
    }

    /**
     * @return array
     */
    protected function getFields()
    {
        if (!$this->fields) {
            $enabledFieldsConfig = array_map('trim', explode(',', $this->settings['fields']['enabled']));
            $mandatoryFieldsConfig = array_map('trim', explode(',', $this->settings['fields']['mandatory']));

            $fields = array('gender', 'firstName', 'lastName', 'company', 'position', 'address', 'zip',
                'city', 'country', 'language', 'email', 'telephone', 'mobile', 'fax', 'description', 'image', 'pdf');

            if ($this->settings['emailAsUsername']) {
                $fields = array_diff($fields, array('email'));
            }

            $enabledFields = $mandatoryFields = array();
            foreach ($fields as $field) {
                if (in_array($field, $enabledFieldsConfig)) {
                    $enabledFields[$field] = $field;
                }
                if (in_array($field, $mandatoryFieldsConfig)) {
                    $mandatoryFields[$field] = $field;
                }
            }

            $this->fields = array(
                'enabled'   => $enabledFields,
                'mandatory' => $mandatoryFields,
            );
        }

        return $this->fields;
    }

    /**
     * @param string $username
     * @param string $password
     * @param boolean $force
     * @return boolean
     */
    protected function loginUser($username, $password, $force = false)
    {
        $check = false;
        $loginData = array(
            'username'      => $username,
            'uident_text'   => $password,
            'status'        => 'login',
        );

        $GLOBALS['TSFE']->fe_user->checkPid = '';
        $info = $GLOBALS['TSFE']->fe_user->getAuthInfoArray();
        $user = $GLOBALS['TSFE']->fe_user->fetchUserRecord($info['db_user'], $loginData['username']);

        if (isset($user) && $user != '') {
            if (ExtensionManagementUtility::isLoaded('saltedpasswords')) {
                $saltedpasswordsService = GeneralUtility::makeInstance(
                    'TYPO3\CMS\Saltedpasswords\SaltedPasswordService');
                $ok = $saltedpasswordsService->compareUident($user, $loginData);
            } else {
                $ok = $GLOBALS['TSFE']->fe_user->compareUident($user, $loginData);
            }

            if ($ok || $force) {
                $GLOBALS['TSFE']->fe_user->createUserSession($user);
                $GLOBALS['TSFE']->fe_user->setAndSaveSessionData('dummy', TRUE);
                $GLOBALS['TSFE']->fe_user->user = $user;
                $check = true;
                $this->user = null;
            } else {
                $check = false;
            }
        }

        return $check;
    }

    /**
     * @return User
     */
    protected function getUser($required = false)
    {
        if ($this->user === null) {
            $this->user = false;
            if ($GLOBALS['TSFE']->fe_user && $GLOBALS['TSFE']->fe_user->user['uid']) {
                $userUid = (int)$GLOBALS['TSFE']->fe_user->user['uid'];
                $this->user = $this->userRepository->findByUid($userUid);
            }

            if ($required && !$this->user) {
                $GLOBALS['TSFE']->pageNotFoundAndExit('', HTTP_STATUS_403);
            }
        }

        return $this->user;
    }
}