<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaUser\ViewHelpers\Format;

/**
 * @package     imia_user
 * @subpackage  ViewHelpers
 * @author      David Frerich <d.frerich@imia.de>
 */
class ValueViewHelper extends \TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper
{
    /**
     * @var array
     */
    protected $hookObjects;

	/**
     * @param string $field
	 * @return string
	 */
	public function render($field = null)
    {
		$value = $this->renderChildren();
        if (is_object($value)) {
            switch (get_class($value)) {
                case 'TYPO3\CMS\Extbase\Persistence\ObjectStorage':
                case 'TYPO3\CMS\Extbase\Persistence\Generic\LazyObjectStorage':
                    $values = array();
                    foreach ($value as $subvalue) {
                        $values[] = (string)$subvalue;
                    }

                    $output = implode(', ', $values);
                    break;
                default:
                    $output = $value;
            }
        } else {
            $output = $value;
        }

        if ($field) {
            switch ($field) {
                case 'gender':
                    $output = $this->translate('be_backend_user_gender_' . $output);
                    break;
            }
        }

        // Hook: getFieldValue
        foreach ($this->getHookObjects() as $hookObject) {
            if (method_exists($hookObject, 'getFieldValue')) {
                $hookObject->getFieldValue($field, $value, $output, $this);
            }
        }

        return $output;
	}

    /**
     * Translate by key
     *
     * @param string $key
     * @param array $arguments
     * @return string
     */
    protected function translate($key, $arguments = array())
    {
        return (string)\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate($key, 'imia_user', $arguments);
    }

    /**
     * @return array
     */
    protected function getHookObjects()
    {
        if (!$this->hookObjects) {
            $this->hookObjects = array();

            if (is_array($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['tx_imiauser']['userBackend'])) {
                foreach ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['tx_imiauser']['userBackend'] as $_classRef) {
                    $this->hookObjects[] = &\TYPO3\CMS\Core\Utility\GeneralUtility::getUserObj($_classRef);
                }
            }
        }

        return $this->hookObjects;
    }
}